%% Load image and process image

function [out, B, Blength, BlengthR, A] =image2dowel(filename,scaleDown, BmaxH, Lb, LbC, unitsTitle)

%%% INPUTS
%%% filename - file location of image
%%% scaleDown - percentage of downsample
%%% BmaxH=60; % Max height of sculpture (cm)
%%% Lb=121.92; % Length of the dowel rods you ordered (121.92 cm -> 4 feet)
%%% LbC=2; % Length subtracted from dowel rod length to account for cutting (cm)

A =imread(filename); % load image into matlab
B = rgb2gray(A); % convert RGB to grayscale
B = double(B)/max(double(B(:))); % normalize the grayscale image and convert to double
[a1, a2]=size(B);

%%% Pull up low parts in image (This could be removed)
for i=1:a1
    for j=1:a2
        if B(i,j)<.4
            B(i,j)=B(i,j)*2;
        end
    end
end

B = double(B)/max(double(B(:))); % normalize the grayscale image and convert to double
B = imresize(B, scaleDown); % .0205
B = medfilt2(B);

figure;
imagesc(B); colormap('jet');
axis image off

%% Convert pixel values to wood length

[y, x]=size(B);
drillDepth=2.54; % Additional length added to dowel rods so that it can be pushed into board (1 inch) 
Blength=B.*BmaxH; % Multiple image matrix by max height to convert image matrix to dowel rod length
Blength=Blength+drillDepth; % Add drill depth
BlengthR=reshape(Blength,x*y,1);

%% Optimize dowel rods for cutting

clear out

Lb=Lb-LbC; % to account for cutting
used=[]; % declare used
j=1;
allInd=1:1:x*y;
cc=1;

disp('. . .  M i N i M i Z i N g   D o W e L s   U s E d . . .')
while length(used)<x*y
    summie=0;
    notUsed= setdiff(allInd,used);
    a=[];
    check=0;
    
    while check<length(notUsed)
        
        ind=notUsed(check+1);
        summie=BlengthR(ind)+summie;
        
        if summie>Lb
            summie=summie-BlengthR(ind);
        else
            used(j)=ind;
            a=[a ind];
            j=j+1;
        end
        check=check+1;
    end
    out{cc}=a;
    cc=cc+1;
end

clc
disp('    SUMMARY');
disp('-----------------');
disp(['Image dimensions: ',num2str(y),' x ' ,num2str(x)]);
disp(['Number of cuts: ',num2str(y*x)]);
disp(['Total length of dowel rod used: ',num2str(round(sum(BlengthR))), ' ' ,unitsTitle]);
disp(['You need to buy ',num2str(length(out)), ' dowel rods with unit length ',num2str(Lb), ' ' ,unitsTitle]);
disp('Click any button to continue');
pause;

%% Check how well algorithm worked. Add up lengths of cut to see if they are close to length of dowel rods ordered
% clear sum S
% for d=1:length(out)
%     S(d)=sum(BlengthR(out{d})); % length of dowels 
% end
% 
% figure;
% plot(S,'ob'); ylim([0 Lb])

